package sun.net.www.http;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import java.util.ArrayList;
import java.security.PrivilegedAction;
import java.net.URL;
import java.security.AccessController;
import sun.security.action.GetIntegerAction;
import java.util.HashMap;

public class KeepAliveCache extends HashMap<KeepAliveKey, ClientVector> implements Runnable
{
    private static final long serialVersionUID = -2937172892064557949L;
    static final int MAX_CONNECTIONS = 5;
    static int result;
    static final int LIFETIME = 5000;
    private Thread keepAliveTimer;
    
    static int getMaxConnections() {
        if (KeepAliveCache.result == -1) {
            KeepAliveCache.result = (int)AccessController.doPrivileged(new GetIntegerAction("http.maxConnections", 5));
            if (KeepAliveCache.result <= 0) {
                KeepAliveCache.result = 5;
            }
        }
        return KeepAliveCache.result;
    }
    
    public KeepAliveCache() {
        this.keepAliveTimer = null;
    }
    
    public synchronized void put(URL url, Object o, HttpClient httpClient) {
        int n = (this.keepAliveTimer == null) ? 1 : 0;
        if (n == 0 && !this.keepAliveTimer.isAlive()) {
            n = 1;
        }
        if (n != 0) {
            this.clear();
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    ThreadGroup threadGroup;
                    ThreadGroup parent;
                    for (threadGroup = Thread.currentThread().getThreadGroup(); (parent = threadGroup.getParent()) != null; threadGroup = parent) {}
                    KeepAliveCache.this.keepAliveTimer = new Thread(threadGroup, KeepAliveCache.this, "Keep-Alive-Timer");
                    KeepAliveCache.this.keepAliveTimer.setDaemon(true);
                    KeepAliveCache.this.keepAliveTimer.setPriority(8);
                    KeepAliveCache.this.keepAliveTimer.setContextClassLoader(null);
                    KeepAliveCache.this.keepAliveTimer.start();
                    return null;
                }
            });
        }
        KeepAliveKey o2 = new KeepAliveKey(url, o);
        ClientVector clientVector = (ClientVector)super.get(o2);
        if (clientVector == null) {
            int keepAliveTimeout = httpClient.getKeepAliveTimeout();
            ClientVector value = new ClientVector((keepAliveTimeout > 0) ? (keepAliveTimeout * 1000) : 5000);
            value.put(httpClient);
            super.put(o2, value);
        }
        else {
            clientVector.put(httpClient);
        }
    }
    
    public synchronized void remove(HttpClient o, Object o2) {
        KeepAliveKey key = new KeepAliveKey(o.url, o2);
        ClientVector clientVector = (ClientVector)super.get(key);
        if (clientVector != null) {
            clientVector.remove(o);
            if (clientVector.empty()) {
                this.removeVector(key);
            }
        }
    }
    
    synchronized void removeVector(KeepAliveKey key) {
        super.remove(key);
    }
    
    public synchronized HttpClient get(URL url, Object o) {
        ClientVector clientVector = (ClientVector)super.get(new KeepAliveKey(url, o));
        if (clientVector == null) {
            return null;
        }
        return clientVector.get();
    }
    
    @Override
    public void run() {
        while (true) {
            try {
                Thread.sleep(5000L);
            }
            catch (InterruptedException ex) {}
            synchronized (this) {
                long currentTimeMillis = System.currentTimeMillis();
                ArrayList o = new ArrayList();
                for (KeepAliveKey keepAliveKey : this.keySet()) {
                    ClientVector clientVector = (ClientVector)super.get(keepAliveKey);
                    synchronized (clientVector) {
                        int i = 0;
                        while (i < clientVector.size()) {
                            KeepAliveEntry keepAliveEntry = (KeepAliveEntry)clientVector.elementAt(i);
                            if (currentTimeMillis - keepAliveEntry.idleStartTime > (long)clientVector.nap) {
                                keepAliveEntry.hc.closeServer();
                                ++i;
                            }
                            else {
                                break;
                            }
                        }
                        clientVector.subList(0, i).clear();
                        if (clientVector.size() == 0) {
                            o.add(keepAliveKey);
                        }
                        else {
                            continue;
                        }
                    }
                }
                Iterator<Object> iterator2 = o.iterator();
                while (iterator2.hasNext()) {
                    this.removeVector((KeepAliveKey)iterator2.next());
                }
            }
            if (this.size() <= 0) {
                break;
            }
            continue;
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        throw new NotSerializableException();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        throw new NotSerializableException();
    }
    
    static {
        KeepAliveCache.result = -1;
    }
}
